/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <config.h>

#include <ctype.h>

#include <gnome.h>

#include <pan/pan-glib-extensions.h>

#include <pan/articlelist.h>
#include <pan/globals.h>
#include <pan/grouplist.h>
#include <pan/gui.h>
#include <pan/gui-paned.h>
#include <pan/prefs.h>
#include <pan/text.h>


void
gui_paned_select_all (void)
{
	switch (gui_paned_get_current_pane ())
	{
		case 0: /* groups tab */
			grouplist_select_all ();
			break;
		case 1: /* articles tab */
			articlelist_select_all ();
			break;
		case 2: /* message tab */
			text_select_all ();
			break;
		default:
			break;
	}
}


/*---[ gui_paned_construct ]------------------------------------------
 * build the paned GUI layout
 *--------------------------------------------------------------------*/
void
gui_paned_construct (void)
{
	GtkWidget * w [3];
	const gchar * pch;
	int i;
	int vpos;
	int hpos;

	/* ref the three main widgets so that reparenting them won't kill them */
	gtk_widget_ref (groups_vbox);
	gtk_widget_ref (articlelist_ctree);
	gtk_widget_ref (text_box);

	/* clear out the workarea */
	if (groups_vbox->parent != NULL)
		gtk_container_remove (GTK_CONTAINER(groups_vbox->parent), groups_vbox);
	if (articlelist_ctree->parent != NULL)
		gtk_container_remove (GTK_CONTAINER(articlelist_ctree->parent), articlelist_ctree);
	if (text_box->parent != NULL)
		gtk_container_remove (GTK_CONTAINER(text_box->parent), text_box);
	gtk_container_foreach (GTK_CONTAINER(contents_vbox), (GtkCallback)gtk_widget_destroy, NULL);


	/**
	***  The layout_str is a four-character string where the first
	***  character is in range [1..5] representing the five layouts
	***  Pan supports.  The remaining three characters show which
	***  widgets are placed first, second, and third, respectively.
	***  The grouplist is represented by 'g', the thread view is
	***  represented by 't', and the article is represented by 'a'.
	**/

	pch = layout_str + 1;
	for (i=0; i<=2; ++i, ++pch) {
		if (tolower(*pch)=='g')
			w[i] = groups_vbox;
		else if (tolower(*pch)=='t')
			w[i] = articlelist_ctree;
		else
			w[i] = text_box;
	}

	vpos = gnome_config_get_int_with_default ("/Pan/Geometry/vpaned=140", NULL);
	hpos = gnome_config_get_int_with_default ("/Pan/Geometry/hpaned=265", NULL);

	/* layout part 2: build the array */
	switch (*layout_str)
	{
		case '1':
		{
			GtkWidget * v = gtk_vpaned_new ();
			GtkWidget * h = gtk_hpaned_new ();
			gtk_paned_add1 (GTK_PANED(h), w[0]);
			gtk_paned_add2 (GTK_PANED(h), w[1]);
			gtk_paned_set_position (GTK_PANED(h), hpos);
			gtk_paned_add1 (GTK_PANED(v), h);
			gtk_paned_add2 (GTK_PANED(v), w[2]);
			gtk_paned_set_position (GTK_PANED(v), vpos);
			gtk_box_pack_start (GTK_BOX(contents_vbox), v, TRUE, TRUE, 0);
			break;
		}
		case '2':
		{
			GtkWidget * v = gtk_vpaned_new ();
			GtkWidget * h = gtk_hpaned_new ();
			gtk_paned_add1 (GTK_PANED(h), w[1]);
			gtk_paned_add2 (GTK_PANED(h), w[2]);
			gtk_paned_set_position (GTK_PANED(h), hpos);
			gtk_paned_add1 (GTK_PANED(v), w[0]);
			gtk_paned_add2 (GTK_PANED(v), h);
			gtk_paned_set_position (GTK_PANED(v), vpos);
			gtk_box_pack_start (GTK_BOX(contents_vbox), v, TRUE, TRUE, 0);
			break;
		}
		case '3':
		{
			GtkWidget * v = gtk_vpaned_new ();
			GtkWidget * h = gtk_hpaned_new ();
			gtk_paned_add1 (GTK_PANED(v), w[0]);
			gtk_paned_add2 (GTK_PANED(v), w[1]);
			gtk_paned_set_position (GTK_PANED(v), vpos);
			gtk_paned_add1 (GTK_PANED(h), v);
			gtk_paned_add2 (GTK_PANED(h), w[2]);
			gtk_paned_set_position (GTK_PANED(h), hpos);
			gtk_box_pack_start (GTK_BOX(contents_vbox), h, TRUE, TRUE, 0);
			break;
		}
		case '5':
		{
			GtkWidget * v1 = gtk_vpaned_new ();
			GtkWidget * v2 = gtk_vpaned_new ();
			gtk_paned_add1 (GTK_PANED(v2), w[1]);
			gtk_paned_add2 (GTK_PANED(v2), w[2]);
			gtk_paned_add1 (GTK_PANED(v1), w[0]);
			gtk_paned_add2 (GTK_PANED(v1), v2);
			gtk_paned_set_position (GTK_PANED(v1), vpos);
			gtk_paned_set_position (GTK_PANED(v2), hpos);
			gtk_box_pack_start (GTK_BOX(contents_vbox), v1, TRUE, TRUE, 0);
			break;
		}
		case '4':
		default:
		{
			GtkWidget * v = gtk_vpaned_new ();
			GtkWidget * h = gtk_hpaned_new ();
			gtk_paned_add1 (GTK_PANED(v), w[1]);
			gtk_paned_add2 (GTK_PANED(v), w[2]);
			gtk_paned_set_position (GTK_PANED(v), vpos);
			gtk_paned_add1 (GTK_PANED(h), w[0]);
			gtk_paned_add2 (GTK_PANED(h), v);
			gtk_paned_set_position (GTK_PANED(h), hpos);
			gtk_box_pack_start (GTK_BOX(contents_vbox), h, TRUE, TRUE, 0);
			break;
		}
	}

	Pan.viewmode = GUI_PANED;
	gui_restore_column_widths (Pan.group_clist, "group");
	gui_restore_column_widths (Pan.article_ctree, "threads");
	gtk_widget_show_all (GTK_WIDGET(contents_vbox));

	gtk_widget_unref (groups_vbox);
	gtk_widget_unref (articlelist_ctree);
	gtk_widget_unref (text_box);
}


/*---[ gui_paned_get_current_pane ]-----------------------------------
 * get the current pane, returns an integer reflecting which pane
 * the user is 'in' ... almost identical to the gtk_notebook_get_page
 * functions
 *--------------------------------------------------------------------*/
int
gui_paned_get_current_pane (void)
{
	if (GTK_WIDGET_HAS_FOCUS (Pan.group_clist))
		return 0;
	if (GTK_WIDGET_HAS_FOCUS (Pan.article_ctree))
		return 1;
	if (GTK_WIDGET_HAS_FOCUS (Pan.text))
		return 2;
	
	return -1;
}


/*--------------------------------------------------------------------
 * ENTER key pressed while in the paned mode, process accordingly
 *--------------------------------------------------------------------*/
void
gui_paned_enter_clicked (void)
{
	switch (gui_paned_get_current_pane())
	{
		case 0: /* group list */
		{
			Group * g = grouplist_get_selected_group ();
			if (g != NULL)
				articlelist_set_group_thread (g);
			break;
		}
		case 1: /* article tree */
		{
			Article* a = articlelist_get_selected_article ();
			if (a != NULL)
				text_set_from_article (a, NNTP_READ, FALSE);
			break;
		}
		case 2: /* message body */
		{
			break;
		}
		default:
		{
			pan_warn_if_reached ();
			break;
		}
	}
}
