/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <config.h>

#include <errno.h>
#include <stdio.h>
#include <string.h>

#include <gnome.h>

#include <pan/debug.h>
#include <pan/file-grouplist.h>
#include <pan/file-headers.h>
#include <pan/log.h>
#include <pan/pan-glib-extensions.h>
#include <pan/server.h>
#include <pan/serverlist.h>
#include <pan/util-file.h>

#include <pan/globals.h>
#include <pan/gui.h>
#include <pan/prefs.h>
#include <pan/serverlist.h>
#include <pan/server-ui.h>
#include <pan/util.h>

/**
***
***  MENU
***
**/

static int server_qty = 0;
static gchar * first_server_name = NULL;
static GnomeUIInfo * server_menu = NULL;
static GnomeUIInfo * server_menu_radiobox = NULL;
static GnomeUIInfo end = GNOMEUIINFO_END;
static GHashTable * server_to_menubutton = NULL;
static gboolean server_activate_registered = FALSE;

static void
server_selected_cb (GtkWidget  * w,
                    Server     * server)
{
	debug_enter ("server_selected_cb");
odebug0 ("server_selected_cb");

	if (GTK_CHECK_MENU_ITEM(w)->active)
		serverlist_set_active_server (server);

	debug_exit ("server_selected_cb");
}

static gint
server_activated_cb (gpointer server, gpointer foo, gpointer bar)
{
	GnomeUIInfo * menu_info = NULL;

	if (server != NULL)
		menu_info = g_hash_table_lookup (server_to_menubutton, server);
	if (menu_info != NULL)
		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menu_info->widget), TRUE);

	return 0;
}

void
server_menu_update (void)
{	
	GSList* l;
	GnomeUIInfo * buf;
	GnomeUIInfo * radio_box;
	GString * str = g_string_new (NULL);
	gchar * path;
	gchar * new_first_server_name = NULL;
	int i;
	debug_enter ("server_menu_update");

	/* make sure we update the menu when a server is activated */
	if (!server_activate_registered)
	{
		server_activate_registered = TRUE;

		pan_callback_add (serverlist_get_server_activated_callback(),
		                  server_activated_cb,
		                  NULL);
	}

	/* build the new menu */
	if (server_to_menubutton != NULL)
		g_hash_table_destroy (server_to_menubutton);
	server_to_menubutton = g_hash_table_new (g_direct_hash, g_direct_equal);
	l = serverlist_get_servers ();
	buf = g_new0 (GnomeUIInfo, g_slist_length(l)+2);
	i = 0;
	if (!l)
	{
		GnomeUIInfo * info = &buf[i];
		info->type = GNOME_APP_UI_ITEM;
		info->label = _("No Servers Defined");
		info->hint = _("No Servers Defined");
		info->moreinfo = NULL;
		info->user_data = NULL;
		++i;
	}
	for (; l!=NULL; l=l->next)
	{
		Server *s = SERVER(l->data);
		GnomeUIInfo * info = buf + i;

		/* skip 'invisible' servers used for folders, etc. */
		if (!strncmp (s->name, INTERNAL_SERVER_NAME, strlen(INTERNAL_SERVER_NAME)))
			continue;

		g_hash_table_insert (server_to_menubutton, s, info);
		info->type = GNOME_APP_UI_ITEM;
		info->label = s->name;
		info->hint = s->name;
		info->moreinfo = server_selected_cb;
		info->user_data = s;
		++i;
		if (new_first_server_name == NULL)
			new_first_server_name = g_strdup (s->name);
	}
	buf[i] = end;

	radio_box = g_new0 (GnomeUIInfo, 2);
	radio_box[0] = radio_box[1] = end;
	radio_box[0].type = GNOME_APP_UI_RADIOITEMS;
	radio_box[0].label = radio_box[0].hint = _("Online");
	radio_box[0].moreinfo = buf;


	/* update the main menu */
	if (is_nonempty_string(first_server_name))
	{
		gchar* pch = g_strdup_printf ("Online/%s", first_server_name);
		gnome_app_remove_menus (GNOME_APP(Pan.window), pch, server_qty);
		g_free (pch);
	}

	path = g_strdup_printf ("%s/%s", _("_Online"), "<Separator>");
	gnome_app_insert_menus (GNOME_APP(Pan.window), path, radio_box);
	g_free (path);

	/* update the local varialbes */
	g_free (first_server_name);
	g_free (server_menu);
	g_free (server_menu_radiobox);
	first_server_name = new_first_server_name;
	server_menu = buf;
	server_menu_radiobox = radio_box;
	server_qty = i;

	/* cleanup */
	g_string_free (str, TRUE);
	debug_exit ("server_menu_update");
}

/**
***
***  ONLINE PROMPT
***
**/

static void
online_prompt_cb (gint reply,
                  gpointer user_data)
{
	const gboolean online = !reply;
	Server * server = SERVER(user_data);
	if (server->online != online)
	{
		server_set_online (SERVER(user_data), online);
		server->online_prompted = FALSE;
	}
}

void
server_prompt_to_go_online (Server* server)
{
	g_return_if_fail (server!=NULL);

	if (!server->online && !server->online_prompted)
	{
		gchar* question;

		server->online_prompted = TRUE;

		question = g_strdup_printf (_("Go Online for Server \"%s\"?"),
		                            server->name);

		pan_lock ();
		gnome_question_dialog_parented (question,
		                                online_prompt_cb,
		                                server,
		                                GTK_WINDOW(Pan.window));
		pan_unlock ();

		g_free (question);
	}
}
