#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <glib.h>

#include <gmime/md5-utils.h>

#include <pan/acache.h>
#include <pan/server.h>
#include <pan/group.h>
#include <pan/article.h>
#include <pan/base-prefs.h>
#include <pan/util-file.h>
#include <pan/status-item.h>
#include <pan/decode.h>

#define remove(A)

static void
check_md5 (const gchar * filename, const guchar * expected_sum)
{
	gint i;
	gchar * full_filename;
	guchar sum [16];
	guchar sum_hex [33];

	/* init */
	memset (sum, '\0', 16);
	memset (sum_hex, '\0', 33);

	/* get the md5sum */
	full_filename = g_strdup_printf ("%s/%s", download_dir, filename);
	md5_get_digest_from_file (full_filename, sum);
	unlink (full_filename);
	g_free (full_filename);
	for (i=0; i<16; ++i)
		sprintf (sum_hex+(i*2), "%02x", sum[i]);
	sum_hex[32] = '\0';

	/* compare */
	if (memcmp(expected_sum, sum_hex, 32)) {
		g_message ("File `%s' failed md5sum test", filename);
		g_message ("expected [%s]", expected_sum);
		g_message ("     got [%s]", sum_hex);
		exit (1);
	}

	return;
}

static gchar*
my_status_item_describe (const StatusItem * item)
{
	return g_strdup ("decode_test");
}

int
main (void)
{
	decode_data dd;
	Server * server;
	Group * group;
	GPtrArray * a;
	StatusItem * status;

	/* setup */
	g_thread_init (NULL);

	/* init panbase */
	if (1) {
		gchar * base;
		gchar * data_dir;
		gchar * temp_dir;
		gchar * download_dir;

		base = getenv ("srcdir");
		base = g_strdup_printf ("%s/", base!=NULL ? base : getenv("PWD"));
		data_dir = g_strdup_printf ("%s/data/", base);
		temp_dir = g_strdup_printf ("%s/data/tmp/", base);
		download_dir = g_strdup_printf ("%s/data/save/", base);
		base_prefs_init (data_dir, temp_dir, download_dir, 1000, FALSE, FALSE);
		g_free (data_dir);
		g_free (temp_dir);
		g_free (download_dir);
	}

	server = server_new ();
	server->name = g_strdup ("Server");
	group = group_new ("alt.test");
	server_add_groups (server, &group, 1, NULL, NULL);
	a = g_ptr_array_new ();
	status = status_item_new (my_status_item_describe);

	/**
	***  Mime Attachment
	**/

	if (1)
	{
		GSList * sl = NULL;
		gchar * text = "two_mime_attachments";
		Article * article = article_new (group);
		article_set_header (article, HEADER_SUBJECT, text, DO_CHUNK);
		article_set_header (article, HEADER_MESSAGE_ID, text, DO_CHUNK);
		article->part = 1;
		article->parts = 1;
		sl = g_slist_append (sl, article);

		dd.server = server;
		dd.group = group;
		dd.articles = sl;
		dd.path = download_dir;
		dd.filename = NULL;
		dd.item = status;
		decode_article (&dd);
		g_slist_foreach (sl, (GFunc)pan_object_unref, NULL);
		g_slist_free (sl);
		sl = NULL;

		check_md5 ("Down-We-Go-1.JPG", "b4bfe57389edb1051f0eb48accbea989");
		check_md5 ("The-Only-Answer-1.JPG", "857f7f9e1fce68414cdf6a110a4ad2d2");
		g_message ("PASS: two mime attachments in message");
	}

	/**
	***  UUEncoded, in Apostrophe Filename
	**/

	if (1)
	{
		GSList * sl = NULL;
		gchar * text = "apostrophe";
		Article * article = article_new (group);
		article_set_header (article, HEADER_SUBJECT, text, DO_CHUNK);
		article_set_header (article, HEADER_MESSAGE_ID, text, DO_CHUNK);
		article->part = 1;
		article->parts = 1;
		sl = g_slist_append (sl, article);

		dd.server = server;
		dd.group = group;
		dd.articles = sl;
		dd.path = download_dir;
		dd.filename = NULL;
		dd.item = status;
		decode_article (&dd);
		g_slist_foreach (sl, (GFunc)pan_object_unref, NULL);
		g_slist_free (sl);
		sl = NULL;

		check_md5 ("ap'str'ph'.txt", "ddcfb039ae385314aa5d8155196bb550");
		g_message ("PASS: filenames with apostrophes");
	}


	/**
	*** "Cut Here"
	*** I hate those posts.
	**/

	if (1)
	{
		const gint parts = 2;
		GSList * sl = NULL;
		guint i;

		for (i=0; i<parts; ++i)
		{
			gchar * text = g_strdup_printf ("noisy_%d_2", i+1);
			Article * article = article_new (group);
			article_set_header (article, HEADER_SUBJECT, text, DO_CHUNK);
			article_set_header (article, HEADER_MESSAGE_ID, text, DO_CHUNK);
			article->part = i+1;
			article->parts = parts;
			sl = g_slist_append (sl, article);
			g_free (text);
		}

		dd.server = server;
		dd.group = group;
		dd.articles = sl;
		dd.path = download_dir;
		dd.filename = NULL;
		dd.item = status;
		decode_article (&dd);
		g_slist_foreach (sl, (GFunc)pan_object_unref, NULL);
		g_slist_free (sl);
		sl = NULL;

		check_md5 ("security_guard.mp3", "09e06d16585567bc269092eb5b515dc5");
		g_message ("PASS: noisy message with `cut here' and end-of-message sig noise");
	}

	/**
	***  This one's so noisy there's even crap INSIDE the encoded text
	**/

	if (1)
	{
		const gint parts = 2;
		GSList * sl = NULL;
		guint i;

		for (i=0; i<parts; ++i)
		{
			gchar * text = g_strdup_printf ("penguin_%d_%d", i+1, parts);
			Article * article = article_new (group);
			article_set_header (article, HEADER_SUBJECT, text, DO_CHUNK);
			article_set_header (article, HEADER_MESSAGE_ID, text, DO_CHUNK);
			article->part = i+1;
			article->parts = parts;
			sl = g_slist_append (sl, article);
			g_free (text);
		}

		dd.server = server;
		dd.group = group;
		dd.articles = sl;
		dd.path = download_dir;
		dd.filename = NULL;
		dd.item = status;
		decode_article (&dd);
		g_slist_foreach (sl, (GFunc)pan_object_unref, NULL);
		g_slist_free (sl);
		sl = NULL;

		check_md5 ("sit3-shine.7.gif", "b2934b889f259ed4e44d4c934e995c00");
		g_message ("PASS: bluespace");
	}

	return 0;
}
