
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <pan/newsrc.h>


int main (void)
{
	gint num;
	Newsrc * n = newsrc_new (NULL, 0, 1000);
	newsrc_init (n, "1-20,100,102,200-300", 0, 1000);

	/* simple test */
	num = 1;
	if (!newsrc_is_article_read(n,1)) return num;
	g_message ("PASS newsrc newsrc_is_article_read");

	/* testing boundaries */
	num = 2;
	if (!newsrc_is_article_read(n,19)) return num;
	if (!newsrc_is_article_read(n,20)) return num;
	if ( newsrc_is_article_read(n,21)) return num;
	g_message ("PASS newsrc newsrc_is_article_read boundaries");

	/* more boundary testing */
	num = 3;
	if (!newsrc_is_article_read(n,100)) return num;
	if ( newsrc_is_article_read(n,101)) return num;
	if (!newsrc_is_article_read(n,102)) return num;
	g_message ("PASS newsrc newsrc_is_article_read boundaries 2");

	/* make sure marking read doesn't corrupt boundaries */
	num = 4;
	newsrc_mark_article (n, 101, TRUE);
	if ( newsrc_is_article_read(n, 99)) return num;
	if (!newsrc_is_article_read(n,100)) return num;
	if (!newsrc_is_article_read(n,101)) return num;
	if (!newsrc_is_article_read(n,102)) return num;
	if ( newsrc_is_article_read(n,103)) return num;
	g_message ("PASS newsrc don't corrupt boundaries when marking read");

	/* make sure marking unread doesn't corrupt boundaries */
	num = 5;
	newsrc_mark_article (n, 101, FALSE);
	if ( newsrc_is_article_read(n, 99)) return num;
	if (!newsrc_is_article_read(n,100)) return num;
	if ( newsrc_is_article_read(n,101)) return num;
	if (!newsrc_is_article_read(n,102)) return num;
	if ( newsrc_is_article_read(n,103)) return num;
	g_message ("PASS newsrc don't corrupt boundaries when marking unread");

	/* mark all read */
	num = 6;
	newsrc_mark_all (n, TRUE);
	if (!newsrc_is_article_read(n, 99)) return num;
	if (!newsrc_is_article_read(n,100)) return num;
	if (!newsrc_is_article_read(n,101)) return num;
	if (!newsrc_is_article_read(n,102)) return num;
	if (!newsrc_is_article_read(n,103)) return num;
	g_message ("PASS newsrc mark all read");

	/* mark all unread */
	num = 7;
	newsrc_mark_all (n, FALSE);
	if ( newsrc_is_article_read(n, 99)) return num;
	if ( newsrc_is_article_read(n,100)) return num;
	if ( newsrc_is_article_read(n,101)) return num;
	if ( newsrc_is_article_read(n,102)) return num;
	if ( newsrc_is_article_read(n,103)) return num;
	g_message ("PASS newsrc mark all unread");

	/* mark it read again & test the read string */
	num = 8;
	newsrc_mark_all (n, TRUE);
	if (!newsrc_is_article_read(n, 99)) return num;
	if (!newsrc_is_article_read(n,100)) return num;
	if (!newsrc_is_article_read(n,101)) return num;
	if (!newsrc_is_article_read(n,102)) return num;
	if (!newsrc_is_article_read(n,103)) return num;
	g_message ("PASS newsrc re-mark all read");

	/* newsrc_get_read_str */
	num = 9;
	newsrc_init (n, "1-20,100,102,200-300", 0, 1000);
	if (strcmp(newsrc_get_read_str(n),"1-20,100,102,200-300")) return num;
	newsrc_mark_article (n, 15, FALSE);
	if (strcmp(newsrc_get_read_str(n),"1-14,16-20,100,102,200-300")) return num;
	newsrc_mark_article (n, 101, TRUE);
	if (strcmp(newsrc_get_read_str(n),"1-14,16-20,100-102,200-300")) return num;
	newsrc_mark_article (n, 103, TRUE);
	if (strcmp(newsrc_get_read_str(n),"1-14,16-20,100-103,200-300")) return num;
	newsrc_mark_article (n, 105, TRUE);
	if (strcmp(newsrc_get_read_str(n),"1-14,16-20,100-103,105,200-300")) return num;
	g_message ("PASS newsrc_get_read_str");

	/* newsrc_set_group_range
	 * note that after a mark all, the lower bound is 0.
	 * this follows newsrc conventions. */
	num = 10;
	newsrc_init (n, "1-20,100,102,200-300", 0, 1000);
	newsrc_set_group_range (n, 15, 220);
	if (strcmp(newsrc_get_read_str(n),"15-20,100,102,200-220")) return num;
	newsrc_set_group_range (n, 50, 219);
	if (strcmp(newsrc_get_read_str(n),"100,102,200-219")) return num;
	newsrc_set_group_range (n, 10, 1000);
	if (strcmp(newsrc_get_read_str(n),"100,102,200-219")) return num;
	newsrc_mark_all (n, TRUE);
	if (strcmp(newsrc_get_read_str(n),"0-1000")) return num;
	g_message ("PASS newsrc_set_group_range");

	/* success */
	return 0;
}
